/// <reference types="react" />
import "./style.css";
export { default as LoFInput } from "./Input";
export { default as SyntaxHighlighting, LoFHighlightStyleSheet, } from "./SyntaxHighlighting";
export { default as parser } from "./parser/parser";
export declare type ExpressionID = number | string;
/**
 * A LoF expression tree.
 * Contains all required data to draw and calculate LoF expressions
 */
export declare type LoFExpressionTree<T> = {
    data: T;
    id: ExpressionID;
    expressions: (LoFExpressionTree<T> | ExpressionID)[];
    depth: number;
    maxDepth: number;
};
/**
 * Creates JSX of nested Laws of Form expressions. Size is based on the parents font-size.
 * Since the reentry drawing happens purely with pseudo divs (::before, ::after),
 * we can only reenter a specific subexpression twice into the form.
 *
 * How to use:
 *
 * <LoF className="custom-class" style={{fontFamily: "cursive"}}>
 *
 * (a)(c)c
 *
 * <\/LoF>
 *
 * @param {object} props
 * @param {LoFExpressionTree<T>}  props.expressionTree - Expression tree of (nested) Laws of Form expressions
 * @param {string}  props.className - Optional React className
 * @param {React.ReactNode} props.children - LoF (Bracket) Expression that needs to be visualized e.g. ((a)(b)c)
 * @param {React.CSSProperties}  props.style - Optional react style sheet for the LoF expression
 * @param {boolean}  props.unwrittenCross - Optional react style sheet for the LoF expression
 * @param {boolean}  props.uc - Optional shortcut for props.unwrittenCross
 */
export default function LoF<T>({ expressionTree, className, style, children, unwrittenCross, uc, }: {
    expressionTree?: LoFExpressionTree<T>;
    className?: string;
    style?: React.CSSProperties;
    children?: React.ReactNode;
    unwrittenCross?: boolean;
    uc?: boolean;
}): JSX.Element;
